@@ -262,7 +262,7 @@ def unzip_path(output_path, zipName):
             os.makedirs(output_path)
     else:
         os.makedirs(output_path)
-    
+
     file = os.path.abspath(zipName)
     if zipfile.is_zipfile(file):
         try:
@@ -547,60 +547,88 @@ def which(program, extra_path=None):
             if program_bak.endswith('.bat') == False:
                 program = program_bak + '.bat'
             exe_file = which_internal(program, extra_path)
-        return exe_file 
+        return exe_file
     else:
         return which_internal(program, extra_path)
 
-def get_mem_info(map_file):
+def get_mem_info(map_file, XIP_ENABLE = True):
+    mem_map_text = ''
+    sym_all_list = []
+    lib_dic_list = []
+    id_list      = []
+    map_lines    = []
     total_ro = 0
     total_rw = 0
-    map_lines = []
+
+    #1. get 'all_map_str' from file
     with open(map_file, 'r') as f:
-        s = f.read()
-        pattern_str = 'Linker script and memory map([\s\S]+?)OUTPUT\(\S*%s\s' % os.path.basename(map_file).replace('.map','.elf')
-        # find memory map (without discard and debug sections)
-        mem_map_list = re.findall(pattern_str, s)
-        mem_map = '' if not mem_map_list else mem_map_list[0]
-        if not mem_map:
-            print('Can\'t parse memory info, memory info get fail!')
+        all_map_str = f.read()
+        if not all_map_str:
+            print ('Can\'t parse map_file!')
             return
 
-        mem_map = mem_map.replace('\r', '')
-        modules = []
-        for l in mem_map.split('\n'):
-            m = re.search(r'0x\w+\s+0x\w+\s+.+?([^/\\]+\.[ao])\((.+\.o)\)', l)
-            if m == None:
-                continue
-            modules.append(m.groups()[0])
-        modules = list(set(modules))
+    #2. get 'sym_map_text' from 'all_map_str'
+    # find memory map (without discard and debug sections)
+    mem_map_list = re.findall(r'Linker script and memory map([\s\S]+?)OUTPUT', all_map_str)
+    mem_map_text = '' if not mem_map_list else mem_map_list[0]
+    if not mem_map_text:
+        print ('Can\'t parse memory info, memory info get fail!')
+        return
+    mem_map_text = mem_map_text.replace('\r', '')
 
-        # find sections address - length in memory map
-        # modules = list(set(item[0] for item in re.findall('0x\w+\s+0x\w+\s+.+?([^/\\\]+\.[ao])(\(.+\.o\))?\r?\n', mem_map)))
-        modules.sort(key = lambda x : x.upper())
+    #3. get all symbol information from 'sym_map_text'
+    sym_all_list_a = re.findall(r' [\.\w]*\.(iram1|text|literal|rodata|rodata1|data|bss|gcc_except_table)(?:\.(\S+)\n? +| +)(0x\w+) +(0x\w+) +.+[/\\](.+\.a)\((.+\.o)\)\n', mem_map_text)
+    sym_all_list_a = map(lambda arg : {'Type':arg[0], 'Sym':arg[1], 'Addr':int(arg[2], 16),
+                    'Size':int(arg[3], 16), 'Lib':arg[4], 'File':arg[5]}, sym_all_list_a)
 
-        for module in modules:
-            module = module.replace('+', r'\+')
+    sym_all_list_o = re.findall(r' [\.\w]*\.(iram1|text|literal|rodata|data|bss|mmu_tbl)(?:\.(\S+)\n? +| +)(0x\w+) +(0x\w+) +.+[/\\](.+\.o)\n', mem_map_text)
+    sym_all_list_o = map(lambda arg : {'Type':arg[0], 'Sym':arg[1], 'Addr':int(arg[2], 16),
+                    'Size':int(arg[3], 16), 'Lib':'null', 'File':arg[4]}, sym_all_list_o)
 
-            # get module's sections's address and size
-            sec_text    = map(lambda arg : {'address':int(arg[0], 16), 'size':int(arg[1], 16)}, re.findall(r'\.text\.\S+\n?\s+(0x\w+)[ \t]+(0x\w+)[ \t]+.+[/\\]'+module+r'(\(.+\.o\))?\r?\n', mem_map))
-            sec_rodata  = map(lambda arg : {'address':int(arg[0], 16), 'size':int(arg[1], 16)}, re.findall(r'\.rodata\.\S+\n?\s+(0x\w+)[ \t]+(0x\w+)[ \t]+.+[/\\]'+module+r'(\(.+\.o\))?\r?\n', mem_map))
-            sec_data    = map(lambda arg : {'address':int(arg[0], 16), 'size':int(arg[1], 16)}, re.findall(r'\.data\.\S+\n?\s+(0x\w+)[ \t]+(0x\w+)[ \t]+.+[/\\]'+module+r'(\(.+\.o\))?\r?\n', mem_map))
-            sec_bss     = map(lambda arg : {'address':int(arg[0], 16), 'size':int(arg[1], 16)}, re.findall(r'\.bss\.\S+\n?\s+(0x\w+)[ \t]+(0x\w+)[ \t]+.+[/\\]'+module+r'(\(.+\.o\))?\r?\n', mem_map))
-            
-            sec_ro      = list(sec_text) + list(sec_rodata);
-            sec_rw      = list(sec_data) + list(sec_bss);
+    sym_com_list_a = re.findall(r' (COMMON) +(0x\w+) +(0x\w+) +.+[/\\](.+\.a)\((.+\.o)\)\n +0x\w+ +(\w+)\n', mem_map_text)
+    sym_com_list_a = map(lambda arg : {'Type':arg[0], 'Sym':arg[5], 'Addr':int(arg[1], 16),
+                    'Size':int(arg[2], 16), 'Lib':arg[3], 'File':arg[4]}, sym_com_list_a)
 
-            ro_size = 0
-            rw_size = 0
-            for sec in sec_ro:
-                ro_size += sec['size']
-            for sec in sec_rw:
-                rw_size += sec['size']
+    sym_com_list_o = re.findall(r' (COMMON) +(0x\w+) +(0x\w+) +.+[/\\](.+\.o)\n +0x\w+ +(\w+)\n', mem_map_text)
+    sym_com_list_o = map(lambda arg : {'Type':arg[0], 'Sym':arg[4], 'Addr':int(arg[1], 16),
+                    'Size':int(arg[2], 16), 'Lib':'null', 'File':arg[3]}, sym_com_list_o)
 
-            total_ro += ro_size
-            total_rw += rw_size
+    sym_all_list.extend(sym_all_list_a)
+    sym_all_list.extend(sym_all_list_o)
+    sym_all_list.extend(sym_com_list_a)
+    sym_all_list.extend(sym_com_list_o)
 
-            map_lines.append(r'| %-40s | %-8d  | %-8d |'%(re.sub('\.[ao]','',module)[:40],ro_size,rw_size))
+    #for each memmap info, classify by mem type
+    for obj_dic in sym_all_list:
+        id_str = obj_dic['Lib']
+        if id_str not in id_list:
+            idx = len(lib_dic_list)
+            lib_dic_list.append({'Lib':obj_dic['Lib'], 'ROM':0, 'RAM':0, 'Text':0, 'Rodata':0, 'Data':0, 'Bss':0})
+            id_list.append(id_str)
+        else:
+            idx = id_list.index(id_str)
+
+        if obj_dic['Type'] == 'text' or obj_dic['Type'] == 'literal' or obj_dic['Type'] == 'iram1':
+            lib_dic_list[idx]['Text'] += obj_dic['Size']
+        elif obj_dic['Type'] == 'rodata' or obj_dic['Type'] == 'rodata1':
+            lib_dic_list[idx]['Rodata'] += obj_dic['Size']
+        elif obj_dic['Type'] == 'data' or obj_dic['Type'] == 'gcc_except_table':
+            lib_dic_list[idx]['Data'] += obj_dic['Size']
+        elif obj_dic['Type'] == 'bss' or obj_dic['Type'] == 'COMMON' or obj_dic['Type'] == 'mmu_tbl':
+            lib_dic_list[idx]['Bss'] += obj_dic['Size']
+
+    lib_dic_list.sort(key = lambda x : x['Lib'].upper())
+
+    #sum ROM and RAM for each library file
+    for lib_dic in lib_dic_list:
+        lib_dic['ROM'] = lib_dic['Text'] + lib_dic['Rodata'] + lib_dic['Data']
+        if XIP_ENABLE == True:
+            lib_dic['RAM'] = lib_dic['Data'] + lib_dic['Bss']
+        else:
+            lib_dic['RAM'] = lib_dic['Text'] + lib_dic['Rodata'] + lib_dic['Data'] + lib_dic['Bss']
+        map_lines.append(r'| %-40s | %-8d  | %-8d |'%(lib_dic['Lib'], lib_dic['ROM'], lib_dic['RAM']))
+        total_rw += lib_dic['RAM']
+        total_ro += lib_dic['ROM']
 
     print('\n                        AOS MEMORY MAP                            ')
     print('|=================================================================|')
