@@ -405,6 +405,62 @@ class YoC:
                 if comp.name == opt_dep["comp_name"]:
                     comp.conditions = opt_dep["condition"]
                     break
+    
+    def doBoardPreConfig(self, board_name=None):
+        # scan all component params
+        for component in self.components:
+            if component.path == os.getcwd():
+                component.load_package()
+                for name in component.depends:
+                    if type(name) == dict:
+                        name = list(name.keys())[0]
+                    c = self.components.get(name)
+                    if c:
+                        c.load_package()
+
+        # run py file or sh file function
+        def run_config_script(script_file, sys_params):
+            params_list = ""
+            # script_file might have args
+            real_file = script_file
+            real_args = ""
+            idx = script_file.find(" --")
+            if idx > 0:
+                real_file = script_file[:idx]
+                real_args = script_file[idx:]
+            if real_file.endswith(".py"):
+                params_list = "python "
+            params_list += "\"%s\" %s %s" % (real_file, sys_params, real_args)
+            # script_process = subprocess.Popen(params_list, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
+            # lines = script_process.stdout.readlines()
+            script_result = os.system(params_list) >> 8
+            if script_result != 0:
+                put_string("Run script '%s' failed" % params_list)
+            else:
+                put_string("Run script '%s' success" % params_list)
+
+            return (script_result==0)
+
+        for component in self.components:
+            if component.path == os.getcwd() and component.type == 'solution':
+                self.solution_component = component
+                if self.solution_component.hw_info.board_name:
+                    self.board_component = self.components.get(self.solution_component.hw_info.board_name)
+                    if not self.board_component:
+                        put_string("111 Not found in `%s` components that the current project depends on. " %
+                                   self.solution_component.hw_info.board_name)
+                        return False
+                    # get board preconfig script and run
+                    # put_string(self.solution_component.hw_info.toolchain_prefix)
+                    if self.board_component.build_config.preconfig_script:
+                        put_string("run user pre config script")
+                        # print(self.board_component.path)
+                        # print(self.board_component.build_config.preconfig_script)
+                        script_file = os.path.join(self.board_component.path, self.board_component.build_config.preconfig_script)
+                        run_config_script(script_file, "")
+                        return True
+                    # 
+        return False
 
     def getSolution(self, board_name=None, compiler='gcc', print_depends=False, exit_if_lost=True):
         for component in self.components:
